/**
 * @file
 * VideoJS player behavior for the player component.
 */
(function (Drupal, once, $) {
  /**
   * Track all VideoJS players on the page.
   */
  let allPlayers = [];

  /**
   * Pauses all other VideoJS players when one starts playing.
   *
   * @param {Object} currentPlayer - The current VideoJS player that is playing.
   */
  function pauseOtherPlayers(currentPlayer) {
    allPlayers.forEach(function (player) {
      // Only pause other players, not the one that's currently playing
      if (player !== currentPlayer && !player.paused()) {
        player.pause();
      }
    });
  }

  /**
   * Registers a VideoJS player in the global tracking list.
   *
   * @param {Object} player - The VideoJS player instance to register.
   */
  function registerPlayer(player) {
    if (
      player &&
      !player.el().hasAttribute('data-videojs-mediablock-processed')
    ) {
      // Add to tracked players list
      allPlayers.push(player);

      // Add play event listener
      player.on('play', function () {
        pauseOtherPlayers(this);
      });

      // Mark as processed
      player.el().setAttribute('data-videojs-mediablock-processed', 'true');
    }
  }

  /**
   * Initialize VideoJS players with one-at-a-time playback behavior.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.videojsMediablockPlayer = {
    attach(context, settings) {
      // Reset the players list when the behavior is first attached
      if (context === document) {
        allPlayers = [];
      }

      // Find all VideoJS players in the context
      const videoElements = context.querySelectorAll('.video-js');

      videoElements.forEach(function (element) {
        // Check if this is a VideoJS player and not already processed
        if (
          element.player &&
          !element.hasAttribute('data-videojs-mediablock-processed')
        ) {
          registerPlayer(element.player);
        }
        // Support for players that might initialize after this behavior runs
        else if (!element.hasAttribute('data-videojs-mediablock-processed')) {
          // Mark for processing
          element.setAttribute('data-videojs-mediablock-waiting', 'true');

          // Use MutationObserver to detect when videojs enhances this element
          const observer = new MutationObserver(function (mutations) {
            if (
              element.player &&
              element.hasAttribute('data-videojs-mediablock-waiting')
            ) {
              registerPlayer(element.player);

              // Mark as processed and clean up
              element.removeAttribute('data-videojs-mediablock-waiting');

              // Disconnect observer once processed
              observer.disconnect();
            }
          });

          // Observe the element for changes
          observer.observe(element, {
            attributes: true,
            childList: true,
            subtree: true,
          });
        }
      });
    },
  };

  // Register callback for when a new VideoJS player is ready
  Drupal.behaviors.videojsMediablockPlayerEvents = {
    attach(context, settings) {
      once('videojs-player-events', 'body', context).forEach(function () {
        // Using Drupal's event system
        $(document).on('videojs-player-ready', function (e, player) {
          registerPlayer(player);
        });
      });
    },
  };

  /**
   * Helper to initialize VideoJS players with SDC components.
   * This is kept outside of Drupal.behaviors as it should run only once.
   */
  if (typeof window.videojs !== 'undefined') {
    Drupal.behaviors.videojsMediablockPlayerInit = {
      attach(context, settings) {
        once('videojs-elements', 'video.video-js', context).forEach(
          function (videoElement) {
            if (typeof window.videojs !== 'undefined') {
              try {
                // Initialize VideoJS on the element
                const player = window.videojs(videoElement, {}, function () {
                  // Player is ready
                  this.el().setAttribute('data-videojs-initialized', 'true');

                  // Trigger a Drupal event
                  $(document).trigger('videojs-player-ready', [this]);
                });
              } catch (e) {
                console.error('Error initializing VideoJS player:', e);
              }
            }
          },
        );
      },
    };
  }
})(Drupal, once, jQuery);
